library seached_list;

import 'package:flutter/material.dart';
import 'package:wowit/Controller/baseController.dart';

import '../util.dart';
import '../filter_widget.dart';

part 'package:wowit/Controller/search_controller.dart';

class SearchedList extends FilterWidget<_SearchController> {
  SearchedList({
    Future<Map<String, List<Map<String, Object>>>> Function() loadData,
    String title,
    List<FilterItem> filterItems,
    this.getOutsideSearchedResultItemView,
    this.scrollController,
    this.getOutSideConditionView,
    this.fieldNameSearching,
    this.searchedListPadding,
  }) : super(
          _SearchController(loadData),
          title: title,
          filterItems: filterItems,
        );

  final ScrollController scrollController;
  final Widget Function(Map<String, Object> dataItem)
      getOutsideSearchedResultItemView;
  final Widget Function(Map<String, Object> mapCondition,
          void Function(String fieldName, Object value) onConditionValueChanged)
      getOutSideConditionView;
  final String fieldNameSearching;
  final EdgeInsets searchedListPadding;

  @override
  Widget getConditionView(Map<String, Object> mapCondition,
      void onValueConditionChanged(String fieldName, Object value)) {
    return getOutSideConditionView(mapCondition, onValueConditionChanged);
  }

  @override
  Widget getMainView(BuildContext context,
      Map<String, List<Map<String, Object>>> data, BaseController controller) {
    final _searchController = controller as _SearchController;
    return NotificationListener(
      onNotification: (notification) {
        if (notification is ScrollUpdateNotification) {
          if (notification.scrollDelta.abs() > 10 &&
              notification.dragDetails != null)
            FocusScope.of(context).unfocus();
        }
        return true;
      },
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Expanded(
            child: _searchController.dataList.isNotEmpty
                ? Padding(
                    padding: searchedListPadding ?? const EdgeInsets.all(0.0),
                    child: ListView.builder(
                      controller: scrollController,
                      itemCount: _searchController.dataList.length,
                      itemBuilder: (BuildContext context, index) {
                        return getOutsideSearchedResultItemView(
                            _searchController.dataList[index]);
                      },
                    ),
                  )
                : this.getEmptyView(),
          ),
        ],
      ),
    );
  }

  void search(String keyword) {
    onConditionValueChanged(fieldNameSearching, keyword);
    applyFilter();
  }
}
