import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:wowit/Controller/baseController.dart';
import 'package:wowit/Controller/filter_controller.dart';
import 'package:wowit/Widget/baseWidgets.dart';
import 'package:wowit/Widget/filter_widget/util.dart';

import 'component/filter_screen.dart';
import 'component/search_screen.dart';

abstract class FilterWidget<T extends BaseController> extends BaseWidget<T> {
  final String title;
  final Color color;
  final FilterController filterController;
  final String fieldNameSearching;
  final EdgeInsets searchedListPadding;
  final Widget filterIcon;

  FilterWidget(
    BaseController baseController, {
    this.fieldNameSearching,
    this.title,
    this.color,
    this.searchedListPadding,
    this.filterIcon,
    @required List<FilterItem> filterItems,
  })  : filterController = FilterController(filterItems),
        super(baseController as T);

  Widget getConditionView(
    Map<String, Object> mapCondition,
    void onValueChange(String fieldName, Object value),
  );

  Widget getLineView(Map<String, Object> dataItem) {
    print('FilterWidget: Need to providea a LineView');
    return SizedBox();
  }

  void _onApply() {
    applyFilter();
    Get.back();
  }

  void onConditionValueChanged(String filterName, Object value) {
    filterController.updateMapCondition(filterName, value);
  }

  void goToFilterView() {
    Get.to(
      () => FilterScreen(
        getConditionView: getConditionView,
        filterController: filterController,
        title: this.title,
        onApply: _onApply,
        onClear: _onClear,
        color: this.color,
      ),
    );
  }

  void goToSearchView() {
    Get.to(
      () => SearchScreen(
        getConditionView: getConditionView,
        filterItems: filterController.filterItems,
        getOutsideSearchedResultItemView: getLineView,
        loadData: getController<T>().loadData,
        title: title,
        fieldNameSearching: fieldNameSearching,
        searchedListPadding: searchedListPadding,
        filterIcon: filterIcon,
      ),
    );
  }

  //Goi tu ben ngoai
  void applyFilter() async {
    final baseController = getController<T>();

    final loadedData = await baseController.loadData();

    final filterDataFieldName = getFilterFieldName(loadedData);

    final filteredData =
        filterController.applyFilter(loadedData[filterDataFieldName]);
    loadedData[filterDataFieldName] = filteredData;
    baseController.setData(loadedData);
    baseController.update();
  }

  //Goi tu ben ngoai
  void applySort(String fieldName, ESortMode sortMode) async {
    final baseController = getController<T>();

    final loadedData = baseController.getData();

    final filterDataFieldName = getFilterFieldName(loadedData);

    final filteredData = filterController.applySort(
        loadedData[filterDataFieldName], fieldName, sortMode);
    loadedData[filterDataFieldName] = filteredData;
    baseController.setData(loadedData);
    baseController.update();
  }

  void _onClear() {
    filterController.clear();
  }

  Object getValueConditionByFieldName(String fieldName) {
    return filterController.mapCondition[fieldName];
  }
}

class FilterItem {
  final String fieldName;

  final EFilterMode filterMode;

  ///Gia tri mac dinh
  ///Nen cung cap gia tri mac dinh khi [filterMode] == [EFilterMode.range]
  final Object defaultValue;

  //Gia tri khi chon 'tat ca'
  //Gap gia tri nay thi se in ra toan bo list ma khong filter.
  final Object allValue;

  const FilterItem({
    @required this.fieldName,
    @required this.filterMode,
    this.defaultValue,
    this.allValue,
  });
}

enum EFilterMode {
  equal,
  range,
  contain,
  lessThan,
  lessOrEqual,
  greaterThan,
  greaterOrEqual,
  insensitiveAndUnsignedContain
}

enum ESortMode {
  smallestToLargest,
  largestToSmallest,
}
